summaryrefslogtreecommitdiff
diff options
context:
space:
mode:
authorArnold D. Robbins <arnold@skeeve.com>2017-07-15 22:27:24 +0300
committerArnold D. Robbins <arnold@skeeve.com>2017-07-15 22:27:24 +0300
commit11d4ea518166ffbc0c2fe85d090723e8f299486c (patch)
tree46159be89127e94393ba7ce83f16caf5d2a6e769
parent4cd70811e7e335fe568f62c564ac2bef0515694f (diff)
downloadgawk-11d4ea518166ffbc0c2fe85d090723e8f299486c.tar.gz
Revert 92ec6835548d36 - don't use DFA if need_start is true.
-rw-r--r--ChangeLog9
-rw-r--r--re.c15
2 files changed, 21 insertions, 3 deletions
diff --git a/ChangeLog b/ChangeLog
index 4c34f193..8c8d91df 100644
--- a/ChangeLog
+++ b/ChangeLog
@@ -1,3 +1,12 @@
+2017-07-15 Arnold D. Robbins <arnold@skeeve.com>
+
+ Revert change of 2016-07-24 that always runs the dfa
+ matcher. Based on a bug report from Alexandre Oliva
+ <lxoliva@fsfla.org>, DFA can cause gawk to hang, even
+ in the C locale.
+
+ * re.c (research): Don't use dfa if need_start is true.
+
2017-07-07 Arnold D. Robbins <arnold@skeeve.com>
Remove warnings from GCC 7.1 compilation.
diff --git a/re.c b/re.c
index 878c884e..d1b0ee7f 100644
--- a/re.c
+++ b/re.c
@@ -268,17 +268,26 @@ research(Regexp *rp, char *str, int start,
rp->pat.not_bol = 1;
/*
* Always do dfa search if can; if it fails, we won't bother
* with the regex search.
*
* The dfa matcher doesn't have a no_bol flag, so don't bother
* trying it in that case.
*
+ * 7/2008: Skip the dfa matcher if need_start. The dfa matcher
+ * has bugs in certain multibyte cases and it's too difficult
+ * to try to special case things.
+ * 7/2017: Apparently there are some cases where DFA gets
+ * stuck, even in the C locale, so we use dfa only if not need_start.
+ *
+ * Should that issue ever get resolved, note this comment:
+ *
* 7/2016: The dfa matcher can't handle a case where searching
* starts in the middle of a string, so don't bother trying it
* in that case.
+ * if (rp->dfa && ! no_bol && start == 0) ...
*/
- if (rp->dfa && ! no_bol && start == 0) {
+ if (rp->dfa && ! no_bol && ! need_start) {
char save;
size_t count = 0;
struct dfa *superset = dfasuperset(rp->dfareg);
@@ -268,17 +268,26 @@ research(Regexp *rp, char *str, int start,
rp->pat.not_bol = 1;
/*
- * Always do dfa search if can; if it fails, we won't bother
- * with the regex search.
+ * Always do dfa search if can; if it fails, then even if
+ * need_start is true, we won't bother with the regex search.
*
* The dfa matcher doesn't have a no_bol flag, so don't bother